/*
 * Copyright (c) 2012-2016. Sencha Inc.
 */

var system = require('system'),
    fs = require('fs');

var args = system.args,
    script = system.args[0].replace(/\\/g, '/'),
    fashionDir = script.substring(0, script.lastIndexOf('/')),
    scssFile = args[1].replace(/\\/g, '/'),
    output = args[2].replace(/\\/g, '/'),
    singleFile = !!args[3],
    exit = function(code, err){
        if (err) {
            console.error((err.stack || err) + '');
        }
        phantom.exit(code || 0);
    };

try {
    var Fashion = require(fashionDir + '/fashion/fashion-phantomjs.js');

    Fashion.Env.isBrowser = false;
    Fashion.Env.isRhino = true;
    Fashion.Env.readFileRhino = function(file) {
        return fs.read(file);
    };

    Fashion.Env.loadFileRhino = function(file, success, error) {
        var content, exception;
        try {
            content = Fashion.Env.readFile(file);
        } catch (err) {
            exception = err;
        }
        if (exception) {
            error(exception);
        } else {
            success(content);
        }
    };

    function build() {
        var builder = new Fashion.Builder({
                context: {
                    libraries: {
                        compass: fashionDir + '/lib/compass/stylesheets/',
                        blueprint: fashionDir + '/lib/blueprint/stylesheets/'
                    }
                }
            }),
            sassFile;
        try {
            sassFile = builder.getSassFile(scssFile);
            sassFile.onReady(function(){
                try {
                    var ast = sassFile.getExpandedAst(),
                        converted = builder.context.convert(ast),
                        func = builder.context.runtime.compile(converted),
                        css = func();

                    css.getText(function(generated){
                        try {
                            if (!Array.isArray(generated) || generated.length === 1 || singleFile) {
                                if (Array.isArray(generated)) {
                                    generated = generated[0];
                                }
                                fs.write(output, generated);
                            }
                            else {
                                var idx = output.lastIndexOf('/'),
                                    baseName = output.substring(0, idx),
                                    fileName = output.substring(idx + 1),
                                    newContent = '';
                                for (var i = 0; i < generated.length; i++) {
                                    var content = generated[i],
                                        newName = fileName.replace(/\.css$/g, '_' + (i + 1) + '.css');
                                    fs.write(baseName + '/' + newName, content);
                                    newContent += "@import '" + newName + "';\n";
                                }
                                fs.write(fileName, newContent);
                            }
                            exit(0);
                        } catch (err) {
                            exit(4, err);
                        }
                    });

                } catch (err) {
                    exit(3, err);
                }
            });
        } catch (err) {
            exit(2, err);
        }
    }

    build();
} catch (err) {
    exit(1, err);
}
